package org.books.common;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

/**
 * <p>
 * CAS Enterprise Application Development mit Java EE (EADJ) - 2007<br/>
 * Modul: Web-Tier (JSF)<br/>
 * Beschreibung:<br/>
 * Diese Klasse initialisiert den Servlet-Context und setzt den Initialisierungs-
 * Parameter das LOG4j (Logging) der Web-Applikation.
 * </p>
 * <p>Copyright (c) 2007 bouquet dot ch, 3032 Hinterkappelen. All rights reseved.</p>
 * @author    Marc Bouquet ( marc at bouquet dot ch, Switzerland )
 * @version   1.0 - 01.05.2007
 * @see       ServletContextListener @link{javax.servlet.ServletContextListener}
 */
public class InitServletContextListener implements ServletContextListener
{
  /**
   * constant for the logging
   */
  private static Logger LOG = Logger.getLogger( InitServletContextListener.class );

  private static final String LOG4J_PATH = "/WEB-INF/log4j.properties";

  /**
   * contextInitialized
   * @param servletContextEvent ServletContextEvent
   */
  public void contextInitialized( ServletContextEvent servletContextEvent )
  {
    ServletContext context = servletContextEvent.getServletContext();
    setInitialAttribute( context, "log4j-path", LOG4J_PATH );
    initLogging( context, "log4j-path" );

    StringBuilder buf = new StringBuilder();
    buf.append( "--- STARTING SERVLET CONTEXT LISTENER ---------------------------------------------------------\n" );
    buf.append( systemDate() );
    buf.append( "\n-----------------------------------------------------------------------------------------------" );
    if ( LOG.isInfoEnabled() )
    {
      LOG.info( "initialized servlet context:\n" + buf.toString() );
    }
  }

  /**
   * Diese Methode sucht ein Servlet-Kontext-Initialisierungsparameter mit einem gegebenen Namen.
   * Falls dieser gefunden wir, wird der Wert in ein Servlet-Kontextattribut desselben Namen eingefuegt.
   * Falls der Wert nicht gefunden wir, wird ein Standard-Wert in das Servlet-Kontextattribut eingefuegt.<br>
   * This method searches a servlet context initialization parameter with a given name.
   * If these found we, the value inserted in a servlet context attribut the same name.
   * If the value not found we, a default value inserted into the servlet context attribut.
   * @param context ServletContext servlet context
   * @param initParamName String context-attribut from web.xml
   * @param defaultValue String defalt-value
   */
  private void setInitialAttribute( ServletContext context, String initParamName, String defaultValue )
  {
    String initValue = context.getInitParameter( initParamName );
    if ( initValue != null )
    {
      context.setAttribute( initParamName, initValue );
    }
    else
    {
      context.setAttribute( initParamName, defaultValue );
    }
  }

  /**
   * Initialize logging.
   * @param context ServletContext servlet context
   * @param log4jPath String context-attribut from web.xml
   */
  private void initLogging( ServletContext context, String log4jPath )
  {
    try
    {
      String initParam = context.getInitParameter( log4jPath );
      int index = initParam.lastIndexOf( ".properties" );
      if ( index > -1 )
      {
        PropertyConfigurator.configure( loadProperties( context.getResource( initParam ) ) );
        PropertyConfigurator.configureAndWatch( initParam );
      }
    }
    catch ( MalformedURLException mfex )
    {
      System.out.println( "InitServletContextListener - MalformedURLException: " + mfex.getMessage() );
    }
    catch ( IOException ioex )
    {
      System.out.println( "InitServletContextListener - IOException: " + ioex.getMessage() );
    }
  }

  /**
   * Diese Methode liest die Log4j-Property-Datei ein und setzt aus dem System-Eigenschaften
   * den Datei-Pfad sofern dieser gesetzt ist.<br/>
   * Beispiel: -Dcommon.client.log4j.url={CATALINA.HOME}\logs\{name}.log<br/>
   * This method reads the log4j-Property-File in and sets from that system properties
   * the file path if these is set.<br/>
   * Example: -Dcommon.client.log4j.url={CATALINA.HOME}\logs\{name}.log
   * @param fileURL URL to log4j.properties file
   * @throws MalformedURLException
   * @throws IOException
   * @return Properties properties
   */
  private Properties loadProperties( URL fileURL ) throws MalformedURLException, IOException
  {
    Properties properties = new Properties();
    properties.load( fileURL.openStream() );
    // load system property
    String systemProperty = System.getProperty( "common.client.log4j.url" );
    if ( systemProperty != null ) // load property from DB
    {
      properties.put( "log4j.appender.ROLE.file", systemProperty );
    }
    checkLogDirectory( (String) properties.get( "log4j.appender.ROLE.file" ) );
    return properties;
  }

  /**
   * Diese Methode ueberprueft Verzeichnis aufgrund des uebergebenen Pfades auf seine Gueltigkeit.<br>
   * This method examines listing due to the handed over path for its validity.
   * @param path String directory path
   */
  private void checkLogDirectory( String path )
  {
    if ( path != null && path.trim().length() > 0 )
    {
      String[] array = path.split( "/" );
      String directory = "";
      for ( int i = 0; i < array.length; i++ )
      {
        if ( array.length - 1 != i )
        {
          array[i] = array[i] + "/";
          directory = directory + array[i];
        }
      }
      File file = new File( directory );
      if ( !file.isDirectory() )
      {
        file.mkdirs();
      }
    }
    else
    {
      System.out.println( "path argument is illegal!" );
    }
  }

  /**
   * contextDestroyed
   * @param servletContextEvent ServletContextEvent
   */
  public void contextDestroyed( ServletContextEvent servletContextEvent )
  {
    StringBuilder buf = new StringBuilder();
    buf.append( "--- END OF SERVLET CONTEXT LISTENER -----------------------------------------------------------\n" );
    buf.append( systemDate() );
    buf.append( "\n-----------------------------------------------------------------------------------------------" );
    if ( LOG.isInfoEnabled() )
    {
      LOG.info( "destroyed servlet context:\n" + buf.toString() );
    }
  }

  /**
   * This method returns the current time.
   * @return String current timestamp
   */
  private String systemDate()
  {
    SimpleDateFormat dateFormat = new SimpleDateFormat( "dd.MM.yyyy - HH:mm:ss - zzzz" );
    return dateFormat.format( new Date( System.currentTimeMillis() ) );
  }
}