package org.books.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.apache.log4j.Logger;
import org.books.data.Book;
import org.books.exception.BookNotFoundException;

/**
 * <p>
 * CAS Enterprise Application Development mit Java EE (EADJ) - 2007<br/> 
 * Modul: Business-Tier (EJB 3)<br/>
 * Uebung: 2 (statless SessionBean)<br/>
 * Beschreibung:<br/> 
 * Diese Klasse (Stateless Session Bean) implementiert die Geschaeftslogik fuer
 * die Katalogsuche.
 * </p> 
 * <p>Copyright (c) 2007 bouquet dot ch, 3032 Hinterkappelen. All rights reseved.</p>
 * @author     Marc Bouquet ( marc at bouquet dot ch, Switzerland )
 * @version    1.0 - 25.06.2007
 * @see CatalogManagerLocal @link{org.books.model.CatalogManagerLocal}
 * @see CatalogManagerRemote @link{org.books.model.CatalogManagerRemote}
 */
@Stateless
public class CatalogManagerBean implements CatalogManagerLocal, CatalogManagerRemote
{
  /**
   * constant for the logging
   */
  private transient Logger LOG = Logger.getLogger( CatalogManagerBean.class );

  /**
   * Entity manager
   */
  @PersistenceContext( unitName = "bookstore" )
  private EntityManager em;

  /**
   * Diese Methode erstellt ein lookup auf ein Buch.
   * @param bookId long
   * @throws BookNotFoundException
   * @return Book
   */
  public Book lookupBook( long bookId ) throws BookNotFoundException
  {
    Book book = (Book) em.createQuery( "from book b where bookId=" + bookId ).getSingleResult();
    if ( book == null )
    {
      throw new BookNotFoundException();
    }
    return book;
  }

  /**
   * Mit dieser Methode kann nach einem Buch gesucht werden.
   * @param title String
   * @param author String
   * @param publisher String
   * @return List<Book>
   */
  public List<Book> searchBooks( String title, String author, String publisher )
  {
    List<Book> resultList = new ArrayList<Book>();
    Map<String, String> tokens = new HashMap<String, String>();
    if ( title != null && title.trim().length() > 0 )
    {
      tokens.put( "title", title.toLowerCase() );
    }
    if ( author != null && author.trim().length() > 0 )
    {
      tokens.put( "authors", author.toLowerCase() );
    }
    if ( publisher != null && publisher.trim().length() > 0 )
    {
      tokens.put( "publisher", publisher.toLowerCase() );
    }

    // no restrictions
    if ( tokens.size() == 0 )
    {
      return resultList;
    }

    StringBuilder query = new StringBuilder( "from Book b where " );
    for ( String field: tokens.keySet() )
    {
      query.append( "lower(b." + field + ")" );
      query.append( " like " );
      query.append( "'%" + tokens.get( field ) + "%' " );
      query.append( "and " );
    }
    String queryString = query.substring( 0, query.length() - 4 );
    if ( LOG.isInfoEnabled() )
    {
      LOG.info( "queryString is: <" + queryString + ">" );
    }

    List queryList = em.createQuery( queryString ).getResultList();
    for ( Object row: queryList )
    {
      resultList.add( (Book) row );
    }
    return resultList;
  }

  /**
   * Diese Methode speichert eine Liste von Büchern in der Datenbank.
   * @param list List<Book>
   */
  public void loadBooks( List<Book> list )
  {
    // load books
    for ( Iterator iter = list.iterator(); iter.hasNext(); )
    {
      Book book = (Book) iter.next();
      em.persist( book );
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug( book.toString() );
      }
    }
  }
}