package org.books.model;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.annotation.Resource;
import javax.ejb.EJBException;
import javax.ejb.Remove;
import javax.ejb.Stateful;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.helpers.Loader;
import org.books.common.CommonConstants;
import org.books.data.Address;
import org.books.data.CreditCard;
import org.books.data.LineItem;
import org.books.data.Order;
import org.books.exception.CreditCardExpiredException;
import org.books.exception.IncompleteConversionalStateException;

/**
 * <p>
 * CAS Enterprise Application Development mit Java EE (EADJ) - 2007<br/> 
 * Modul: Business-Tier (EJB 3)<br/>
 * Uebung: 2 (stateful SessionBean)<br/>
 * Beschreibung:<br/> 
 * Diese Klasse (Stateful Session Bean) implementiert die Geschaeftslogik fuer
 * die Abwicklung der Bestellungen.
 * </p> 
 * <p>Copyright (c) 2007 bouquet dot ch, 3032 Hinterkappelen. All rights reseved.</p>
 * @author     Marc Bouquet ( marc at bouquet dot ch, Switzerland )
 * @version    1.0 - 29.06.2007
 * @see OrderManagerLocal @link{org.books.model.OrderManagerLocal}
 * @see OrderManagerRemote @link{org.books.model.OrderManagerRemote}
 */
@Stateful
public class OrderManagerBean implements OrderManagerLocal, OrderManagerRemote
{
  /**
   * constant for the logging
   */
  private transient Logger LOG = Logger.getLogger( OrderManagerBean.class );

  @PersistenceContext( unitName = "bookstore" )
  private EntityManager em;

  @Resource( mappedName = "QueueConnectionFactory" )
  // @Resource( mappedName = "java:/JmsXA" ) // TODO: transaktionsfaehige ConnectionFactory im JBoss
  private ConnectionFactory connectionFactory;

  @Resource( mappedName = "queue/bookstore-OrderQueue" )
  private Destination destination;

  private Address address;

  private CreditCard creditCard;

  /**
   * Diese Methode fügt eine Adresse ein.
   * @param address
   */
  public void setAddress( Address address )
  {
    this.address = address;
  }

  /**
   * Diese Methode fuegt die Kreditkarten-Daten ein.
   * @param creditCard
   * @throws CreditCardExpiredException
   */
  public void setCreditCard( CreditCard creditCard ) throws CreditCardExpiredException
  {
    Calendar expiration = new GregorianCalendar();
    expiration.setTime( creditCard.getExpiration() );
    expiration.add( Calendar.MONTH, 1 );
    Calendar today = new GregorianCalendar();
    if ( expiration.before( today ) )
    {
      LOG.error( "Kreditkarte ist ungueltig <" + expiration + ">" );
      throw new CreditCardExpiredException();
    }
    this.creditCard = creditCard;
  }

  /**
   * Diese Methode erzeugt eine Bestellung aufgrund der Liste.
   * @param items List<LineItem>
   * @return Order
   * @throws IncompleteConversionalStateException
   */
  public Order orderBooks( List<LineItem> items ) throws IncompleteConversionalStateException
  {
    if ( this.address == null )
    {
      LOG.error( "IncompleteConversionalStateException is: <no address found>" );
      throw new IncompleteConversionalStateException( CommonConstants.ERR_NO_ADDRESS );
    }

    if ( this.creditCard == null )
    {
      LOG.error( "IncompleteConversionalStateException is: <no credit card found>" );
      throw new IncompleteConversionalStateException( CommonConstants.ERR_NO_CREDITCARD );
    }

    Order order = new Order();
    order.setAddress( clone( address ) );
    order.setCreditCard( clone( creditCard ) );
    order.setDate( new Date() );

    //    long count = (Long) em.createQuery( "select count(o) from order o" ).getSingleResult();
    //    LOG.debug( "### count is: <" + count + ">" );
    // TODO: bessere loesung suchen fuer einen schluessel
    System.out.println( "### OrderId is: <" + order.getOrderId() + ">" );
    order.setNumber( new Integer( (int) System.currentTimeMillis() ) );
    order.setNumber( new Integer( (int) System.currentTimeMillis() ) );
    order.setItems( items );
    order.setStatus( Order.Status.Open );
    double amount = 0.0;
    for ( LineItem item: items )
    {
      amount += ( item.getQuantity() * item.getBook().getPrice() );
    }
    order.setAmount( amount );
    em.persist( order );
    em.flush();
    System.out.println( "### OrderId is: <" + order.getOrderId() + ">" );

    try
    {
      // TODO: in eigene Methode auslagern
      // senden einer Meldung in die JMS-Queue
      Connection connection = this.connectionFactory.createConnection();
      Session session = connection.createSession( true, 0 );
      MessageProducer producer = session.createProducer( this.destination );

      MapMessage msg = session.createMapMessage();
      msg.setInt( "order.number", order.getNumber() );
      producer.send( msg );
      connection.close();
    }
    catch ( JMSException jmsex )
    {
      LOG.error( "JMSException is: " + jmsex.getMessage(), jmsex );
      throw new EJBException( jmsex );
    }
    return order;
  }

  /**
   * Diese Methode beendet ein Bestellvorgang
   */
  @Remove
  public void dispose()
  {
    // Don't do anything
  }

  /**
   * Clone ein Objekt
   * @return T
   */
  private <T> T clone( T object )
  {
    try
    {
      ByteArrayOutputStream os = new ByteArrayOutputStream();
      new ObjectOutputStream( os ).writeObject( object );
      ByteArrayInputStream in = new ByteArrayInputStream( os.toByteArray() );
      return (T) new ObjectInputStream( in ).readObject();
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      return object;
    }
  }
}