package org.books.model;

import javax.annotation.Resource;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.EJBException;
import javax.ejb.MessageDriven;
import javax.ejb.Timeout;
import javax.ejb.Timer;
import javax.ejb.TimerService;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.apache.log4j.Logger;
import org.books.data.Order;

/**
 * <p>
 * CAS Enterprise Application Development mit Java EE (EADJ) - 2007<br/> 
 * Modul: Business-Tier (EJB 3)<br/>
 * Uebung: 3 (Message Driven Bean)<br/>
 * Beschreibung:<br/>
 * Diese Klasse (Message Driven Bean) implementiert einen MessageListener fuer den 
 * Verarbeitungsprozess einer Bestellung.
 * </p> 
 * <p>Copyright (c) 2007 bouquet dot ch, 3032 Hinterkappelen. All rights reseved.</p>
 * @author     Marc Bouquet ( marc at bouquet dot ch, Switzerland )
 * @version    1.0 - 29.06.2007
 * @see MessageListener @link{javax.jms.MessageListener}
 */
@MessageDriven( activationConfig = { @ActivationConfigProperty( propertyName = "destinationType", propertyValue = "javax.jms.Queue" ),
    @ActivationConfigProperty( propertyName = "acknowledgeMode", propertyValue = "Auto-ackonowledge" ),
    @ActivationConfigProperty( propertyName = "destination", propertyValue = "queue/bookstore-OrderQueue" ) } )
public class OrderProcessor implements MessageListener
{
  /**
   * constant for the logging
   */
  private transient Logger LOG = Logger.getLogger( OrderProcessor.class );
  
  /**
   * The process time of an order
   */
  private static final long ORDER_PROCESS_TIME = 60000;

  @PersistenceContext( unitName = "bookstore" )
  private EntityManager em;

  @Resource
  TimerService timerService;

  /**
   * Diese Methode inizialisiert einen Timer fuer die 
   * Verarbeitung des Bestell-Prozesses.
   * @param msg Message
   */
  public void onMessage( Message msg )
  {
    MapMessage orderProcessMessage = (MapMessage) msg;
    try
    {
      // liest die order.number aus der Meldung
      Integer orderNumber = orderProcessMessage.getInt( "order.number" );
      // setzt den Timer für das Schliessen der Bestellung in 1 Minute
      this.timerService.createTimer( ORDER_PROCESS_TIME, orderNumber );
    }
    catch ( JMSException jmsex )
    {
      LOG.error( "JMSException is: " + jmsex.getMessage(), jmsex );
      throw new EJBException( jmsex );
    }
  }

  /**
   * Diese Methode verarbeitet eine Bestellung
   * @param timer
   */
  @Timeout
  public void processOrder( Timer timer )
  {
    // Setzen der Bestellnummer fuer den Timer
    Number number = (Number) timer.getInfo();

    // Suchen der Bestellung aufgrund der Nummer
    Order order = (Order) em.createQuery( "from Order o where o.number=" + number ).getSingleResult();

    // pruefen ob die Bestellung vorhanden ist und noch den Status offen hat
    if ( order != null && order.getStatus() == Order.Status.Open )
    {
      order.setStatus( Order.Status.Closed );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( "Bestellung <" + number + "> wurde abgeschlossen!" );
      }
    }
  }
}
